@@ -2,7 +2,7 @@
  *
  * rasqal_general.c - Rasqal library startup, shutdown and factories
  *
- * Copyright (C) 2004-2010, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2004-2012, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -40,6 +40,9 @@
 #include "rasqal.h"
 #include "rasqal_internal.h"
 
+#ifdef MAINTAINER_MODE
+#include <git-version.h>
+#endif
 
 /* prototypes for helper functions */
 static void rasqal_delete_query_language_factories(rasqal_world*);
@@ -48,9 +51,9 @@ static void rasqal_free_query_language_factory(rasqal_query_language_factory *fa
 
 /* statics */
 
-const char * const rasqal_short_copyright_string = "Copyright 2003-2011 David Beckett.  Copyright 2003-2005 University of Bristol";
+const char * const rasqal_short_copyright_string = "Copyright 2003-2013 David Beckett.  Copyright 2003-2005 University of Bristol";
 
-const char * const rasqal_copyright_string = "Copyright (C) 2003-2011 David Beckett - http://www.dajobe.org/\nCopyright (C) 2003-2005 University of Bristol - http://www.bristol.ac.uk/";
+const char * const rasqal_copyright_string = "Copyright (C) 2003-2013 David Beckett - http://www.dajobe.org/\nCopyright (C) 2003-2005 University of Bristol - http://www.bristol.ac.uk/";
 
 const char * const rasqal_license_string = "LGPL 2.1 or newer, GPL 2 or newer, Apache 2.0 or newer.\nSee http://librdf.org/rasqal/LICENSE.html for full terms.";
 
@@ -193,13 +196,6 @@ rasqal_world_open(rasqal_world *world)
     return rc;
 #endif
 
-#ifdef RASQAL_QUERY_RDQL
-  rc = rasqal_init_query_language_rdql(world);
-  if(rc)
-    return rc;
-#endif
-
-
   rc = rasqal_raptor_init(world);
   if(rc)
     return rc;
@@ -468,6 +464,7 @@ rasqal_world_get_query_language_description(rasqal_world* world,
 }
 
 
+#ifndef RASQAL_DISABLE_DEPRECATED
 /**
  * rasqal_languages_enumerate:
  * @world: rasqal_world object
@@ -508,11 +505,11 @@ rasqal_languages_enumerate(rasqal_world *world,
     *label = factory->desc.label;
 
   if(uri_string && factory->desc.uri_strings)
-    *uri_string = (const unsigned char*)factory->desc.uri_strings[0];
+    *uri_string = RASQAL_GOOD_CAST(const unsigned char*, factory->desc.uri_strings[0]);
 
   return 0;
 }
-
+#endif
 
 /**
  * rasqal_language_name_check:
@@ -592,13 +589,9 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
   if(level == RAPTOR_LOG_LEVEL_NONE)
     return;
 
-#if RAPTOR_VERSION >= 20005
   buffer = NULL;
   if(raptor_vasprintf(&buffer, message, arguments) < 0)
     buffer = NULL;
-#else
-  buffer = raptor_vsnprintf(message, arguments);
-#endif
 
   if(!buffer) {
     if(locator) {
@@ -705,6 +698,8 @@ rasqal_basename(const char *name)
 }
 
 
+const raptor_unichar rasqal_unicode_max_codepoint = 0x10FFFF;
+
 /**
  * rasqal_escaped_name_to_utf8_string:
  * @src: source name string
@@ -747,7 +742,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
 
     if(c > 0x7f) {
       /* just copy the UTF-8 bytes through */
-      size_t unichar_len = raptor_unicode_utf8_string_get_char((const unsigned char*)p, len+1, NULL);
+      size_t unichar_len = raptor_unicode_utf8_string_get_char(RASQAL_GOOD_CAST(const unsigned char*, p), len + 1, NULL);
       if(unichar_len > len) {
         if(error_handler)
           error_handler(error_data, "UTF-8 encoding error at character %d (0x%02X) found.", c, c);
@@ -793,7 +788,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
           return 0;
         }
         
-        n=sscanf((const char*)p, ((ulen == 4) ? "%04lx" : "%08lx"), &unichar);
+        n = sscanf(RASQAL_GOOD_CAST(const char*, p), ((ulen == 4) ? "%04lx" : "%08lx"), &unichar);
         if(n != 1) {
           if(error_handler)
             error_handler(error_data, "Bad %c escape", c);
@@ -839,9 +834,9 @@ rasqal_uri_init(rasqal_world* world)
   if(!world->rdf_namespace_uri)
     goto oom;
 
-  world->rdf_first_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"first");
-  world->rdf_rest_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"rest");
-  world->rdf_nil_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"nil");
+  world->rdf_first_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, RASQAL_GOOD_CAST(const unsigned char*, "first"));
+  world->rdf_rest_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, RASQAL_GOOD_CAST(const unsigned char*, "rest"));
+  world->rdf_nil_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, RASQAL_GOOD_CAST(const unsigned char*, "nil"));
   if(!world->rdf_first_uri || !world->rdf_rest_uri || !world->rdf_nil_uri)
     goto oom;
 
@@ -995,10 +990,10 @@ rasqal_world_default_generate_bnodeid_handler(void *user_data,
   if(world->default_generate_bnodeid_handler_prefix) {
     memcpy(buffer, world->default_generate_bnodeid_handler_prefix,
            world->default_generate_bnodeid_handler_prefix_length);
-    sprintf((char*)buffer + world->default_generate_bnodeid_handler_prefix_length,
+    sprintf(RASQAL_GOOD_CAST(char*, buffer) + world->default_generate_bnodeid_handler_prefix_length,
             "%d", id);
   } else 
-    sprintf((char*)buffer, "bnodeid%d", id);
+    sprintf(RASQAL_GOOD_CAST(char*, buffer), "bnodeid%d", id);
 
   return buffer;
 }
