@@ -51,7 +51,7 @@ static void rasqal_rowsource_print_header(rasqal_rowsource* rowsource, FILE* fh)
  * @query: query object
  * @user_data: pointer to context information to pass in to calls
  * @handler: pointer to handler methods
- * @vars_table: variables table to use for rows
+ * @vars_table: variables table to use for rows (or NULL)
  * @flags: 0 (none defined so far)
  *
  * Create a new rowsource over a user-defined handler.
@@ -68,14 +68,13 @@ rasqal_new_rowsource_from_handler(rasqal_world* world,
 {
   rasqal_rowsource* rowsource;
 
-  if(!handler)
+  if(!world || !handler)
     return NULL;
 
   if(handler->version < 1 || handler->version > 1)
     return NULL;
 
-  rowsource = (rasqal_rowsource*)RASQAL_CALLOC(rasqal_rowsource, 1,
-                                               sizeof(rasqal_rowsource));
+  rowsource = RASQAL_CALLOC(rasqal_rowsource*, 1, sizeof(*rowsource));
   if(!rowsource) {
     if(handler->finish)
       handler->finish(NULL, user_data);
@@ -88,19 +87,17 @@ rasqal_new_rowsource_from_handler(rasqal_world* world,
   rowsource->handler = handler;
   rowsource->flags = flags;
 
-  rowsource->size= -1;
+  rowsource->size = -1;
 
+  rowsource->generate_group = 0;
+  
   if(vars_table)
     rowsource->vars_table = rasqal_new_variables_table_from_variables_table(vars_table);
   else
     rowsource->vars_table = NULL;
 
-  /* no free method here - the variables are owned by rowsource->vars_table */
-#ifdef HAVE_RAPTOR2_API
-  rowsource->variables_sequence = raptor_new_sequence(NULL, (raptor_data_print_handler)rasqal_variable_print);
-#else
-  rowsource->variables_sequence = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
-#endif
+  rowsource->variables_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                                      (raptor_data_print_handler)rasqal_variable_print);
   if(!rowsource->variables_sequence) {
     rasqal_free_rowsource(rowsource);
     return NULL;
@@ -125,7 +122,8 @@ rasqal_new_rowsource_from_handler(rasqal_world* world,
 void
 rasqal_free_rowsource(rasqal_rowsource *rowsource)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(rowsource, rasqal_rowsource);
+  if(!rowsource)
+    return;
 
   if(rowsource->handler->finish)
     rowsource->handler->finish(rowsource, rowsource->user_data);
@@ -157,11 +155,15 @@ int
 rasqal_rowsource_add_variable(rasqal_rowsource *rowsource, rasqal_variable* v)
 {
   int offset;
-  
+
+  if(!rowsource || !v)
+    return -1;
+
   offset = rasqal_rowsource_get_variable_offset_by_name(rowsource, v->name);
   if(offset >= 0)
     return offset;
 
+  v = rasqal_new_variable_from_variable(v);
   if(raptor_sequence_push(rowsource->variables_sequence, v))
     return -1;
 
@@ -199,7 +201,8 @@ rasqal_rowsource_ensure_variables(rasqal_rowsource *rowsource)
 
 #ifdef RASQAL_DEBUG
   if(!rc) {
-    RASQAL_DEBUG2("rowsource %p ", rowsource);
+    RASQAL_DEBUG3("%s rowsource %p header: ", rowsource->handler->name,
+                  rowsource);
     rasqal_rowsource_print_header(rowsource, stderr);
   }
 #endif
@@ -223,34 +226,82 @@ rasqal_rowsource_read_row(rasqal_rowsource *rowsource)
 {
   rasqal_row* row = NULL;
   
-  if(rowsource->finished)
+  if(!rowsource || rowsource->finished)
     return NULL;
 
-  if(rasqal_rowsource_ensure_variables(rowsource))
-    return NULL;
+  if(rowsource->flags & RASQAL_ROWSOURCE_FLAGS_SAVED_ROWS) {
+    /* return row from saved rows sequence at offset */
+    row = (rasqal_row*)raptor_sequence_get_at(rowsource->rows_sequence,
+                                              rowsource->offset++);
+#ifdef RASQAL_DEBUG
+    RASQAL_DEBUG3("%s rowsource %p returned saved row:  ",
+                  rowsource->handler->name, rowsource);
+    if(row)
+      rasqal_row_print(row, stderr);
+    else
+      fputs("NONE", stderr);
+    fputs("\n", stderr);
+#endif      
+    if(row)
+      row = rasqal_new_row_from_row(row);
+    /* row is owned by us */
+  } else {
+    if(rasqal_rowsource_ensure_variables(rowsource))
+      return NULL;
 
-  if(rowsource->handler->read_row)
-    row = rowsource->handler->read_row(rowsource, rowsource->user_data);
-  else {
-    if(!rowsource->rows_sequence) {
-      rowsource->rows_sequence = rasqal_rowsource_read_all_rows(rowsource);
-      rowsource->offset = 0;
+    if(rowsource->handler->read_row) {
+      row = rowsource->handler->read_row(rowsource, rowsource->user_data);
+      /* row is owned by us */
+
+      if(row && rowsource->flags & RASQAL_ROWSOURCE_FLAGS_SAVE_ROWS) {
+        if(!rowsource->rows_sequence) {
+          rowsource->rows_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row,
+                                                         (raptor_data_print_handler)rasqal_row_print);
+          rowsource->offset = 0;
+        }
+        /* copy to save it away */
+        row = rasqal_new_row_from_row(row);
+        raptor_sequence_push(rowsource->rows_sequence, row);
+      }
+    } else {
+      if(!rowsource->rows_sequence) {
+        raptor_sequence* seq;
+        
+        seq = rasqal_rowsource_read_all_rows(rowsource);
+        if(rowsource->rows_sequence)
+          raptor_free_sequence(rowsource->rows_sequence);
+        /* rows_sequence now owns all rows */
+        rowsource->rows_sequence = seq;
+
+        rowsource->offset = 0;
+      }
+      
+      if(rowsource->rows_sequence) {
+        /* return row from sequence at offset */
+        row = (rasqal_row*)raptor_sequence_get_at(rowsource->rows_sequence,
+                                                  rowsource->offset++);
+        if(row)
+          row = rasqal_new_row_from_row(row);
+        /* row is owned by us */
+      }
     }
-
-    if(rowsource->rows_sequence)
-      /* remove and return row from sequence at offset */
-      row = (rasqal_row*)raptor_sequence_delete_at(rowsource->rows_sequence,
-                                                   rowsource->offset++);
   }
   
-  if(!row)
+  if(!row) {
     rowsource->finished = 1;
-  else
+    if(rowsource->flags & RASQAL_ROWSOURCE_FLAGS_SAVE_ROWS)
+      rowsource->flags |= RASQAL_ROWSOURCE_FLAGS_SAVED_ROWS;
+  } else {
     rowsource->count++;
 
+    /* Generate a group around all rows if there are no groups returned */
+    if(rowsource->generate_group && row->group_id < 0)
+      row->group_id = 0;
+  }
+
 #ifdef RASQAL_DEBUG
-  RASQAL_DEBUG3("rowsource %p read %s row : ", rowsource, 
-                rowsource->handler->name);
+  RASQAL_DEBUG3("%s rowsource %p returned row:  ", rowsource->handler->name, 
+                rowsource);
   if(row)
     rasqal_row_print(row, stderr);
   else
@@ -268,11 +319,14 @@ rasqal_rowsource_read_row(rasqal_rowsource *rowsource)
  *
  * Get number of rows seen from a rowsource.
  *
- * Return value: row count
+ * Return value: row count or < 0 on failure
  **/
 int
 rasqal_rowsource_get_rows_count(rasqal_rowsource *rowsource)
 {
+  if(!rowsource)
+    return -1;
+
   return rowsource->count;
 }
 
@@ -293,39 +347,81 @@ rasqal_rowsource_read_all_rows(rasqal_rowsource *rowsource)
 {
   raptor_sequence* seq;
 
+  if(!rowsource)
+    return NULL;
+
+  if(rowsource->flags & RASQAL_ROWSOURCE_FLAGS_SAVED_ROWS) {
+    raptor_sequence* new_seq;
+
+    /* Return a complete copy of all previously saved rows */
+    seq = rowsource->rows_sequence;
+    new_seq = rasqal_row_sequence_copy(seq);
+    RASQAL_DEBUG4("%s rowsource %p returning a sequence of %d saved rows\n",
+                  rowsource->handler->name, rowsource,
+                  raptor_sequence_size(new_seq));
+    return new_seq;
+  }
+
+  /* Execute */
   if(rasqal_rowsource_ensure_variables(rowsource))
     return NULL;
 
   if(rowsource->handler->read_all_rows) {
     seq = rowsource->handler->read_all_rows(rowsource, rowsource->user_data);
-    if(!seq)
-#ifdef HAVE_RAPTOR2_API
+    if(!seq) {
       seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row,
                                 (raptor_data_print_handler)rasqal_row_print);
-#else
-      seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row,
-                                (raptor_sequence_print_handler*)rasqal_row_print);
-#endif
-    return seq;
+    } else if(rowsource->generate_group) {
+      int i;
+      rasqal_row* row;
+      
+      /* Set group for all rows if there are no groups returned */
+
+      for(i = 0; (row = (rasqal_row*)raptor_sequence_get_at(seq, i)); i++) {
+        /* if first row has a group ID, end */
+        if(!i && row->group_id >= 0)
+          break;
+        
+        row->group_id = 0;
+      }
+    }
+
+    goto done;
   }
 
-#ifdef HAVE_RAPTOR2_API
   seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row,
                             (raptor_data_print_handler)rasqal_row_print);
-#else
-  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row,
-                            (raptor_sequence_print_handler*)rasqal_row_print);
-#endif
   if(!seq)
     return NULL;
-  
+
   while(1) {
     rasqal_row* row = rasqal_rowsource_read_row(rowsource);
     if(!row)
       break;
+
+    /* Generate a group around all rows if there are no groups returned */
+    if(rowsource->generate_group && row->group_id < 0)
+      row->group_id = 0;
+
     raptor_sequence_push(seq, row);
   }
+
+  done:
+  if(seq && rowsource->flags & RASQAL_ROWSOURCE_FLAGS_SAVE_ROWS) {
+    raptor_sequence* new_seq;
+
+    /* Save a complete copy of all rows */
+    new_seq = rasqal_row_sequence_copy(seq);
+    RASQAL_DEBUG4("%s rowsource %p saving a sequence of %d rows\n",
+                  rowsource->handler->name, rowsource,
+                  raptor_sequence_size(new_seq));
+    rowsource->rows_sequence = new_seq;
+    rowsource->flags |= RASQAL_ROWSOURCE_FLAGS_SAVED_ROWS;
+  }
   
+  RASQAL_DEBUG4("%s rowsource %p returning a sequence of %d rows\n",
+                rowsource->handler->name, rowsource,
+                raptor_sequence_size(seq));
   return seq;
 }
 
@@ -334,11 +430,14 @@ rasqal_rowsource_read_all_rows(rasqal_rowsource *rowsource)
  * rasqal_rowsource_get_size:
  * @rowsource: rasqal rowsource
  *
- * Get rowsource row width
+ * Get rowsource row width or <0 on failure
  **/
 int
 rasqal_rowsource_get_size(rasqal_rowsource *rowsource)
 {
+  if(!rowsource)
+    return -1;
+
   rasqal_rowsource_ensure_variables(rowsource);
 
   return rowsource->size;
@@ -357,6 +456,9 @@ rasqal_rowsource_get_size(rasqal_rowsource *rowsource)
 rasqal_variable*
 rasqal_rowsource_get_variable_by_offset(rasqal_rowsource *rowsource, int offset)
 {
+  if(!rowsource)
+    return NULL;
+
   rasqal_rowsource_ensure_variables(rowsource);
 
   if(!rowsource->variables_sequence)
@@ -374,7 +476,7 @@ rasqal_rowsource_get_variable_by_offset(rasqal_rowsource *rowsource, int offset)
  *
  * Get the offset of a variable into the list of variables
  *
- * Return value: offset or <0 if not present
+ * Return value: offset or <0 if not present or on failure
  **/
 int
 rasqal_rowsource_get_variable_offset_by_name(rasqal_rowsource *rowsource,
@@ -383,6 +485,9 @@ rasqal_rowsource_get_variable_offset_by_name(rasqal_rowsource *rowsource,
   int offset= -1;
   int i;
   
+  if(!rowsource)
+    return -1;
+
   rasqal_rowsource_ensure_variables(rowsource);
 
   if(!rowsource->variables_sequence)
@@ -391,7 +496,8 @@ rasqal_rowsource_get_variable_offset_by_name(rasqal_rowsource *rowsource,
   for(i=0; i < raptor_sequence_size(rowsource->variables_sequence); i++) {
     rasqal_variable* v;
     v = (rasqal_variable*)raptor_sequence_get_at(rowsource->variables_sequence, i);
-    if(!strcmp((const char*)v->name, (const char*)name)) {
+    if(!strcmp(RASQAL_GOOD_CAST(const char*, v->name),
+               RASQAL_GOOD_CAST(const char*, name))) {
       offset = i;
       break;
     }
@@ -446,7 +552,7 @@ rasqal_rowsource_print_header(rasqal_rowsource* rowsource, FILE* fh)
     if(i > 0)
       fputs(", ", fh);
     if(name)
-      fputs((const char*)name, fh);
+      fputs(RASQAL_GOOD_CAST(const char*, name), fh);
     else
       fputs("NULL", fh);
   }
@@ -497,17 +603,14 @@ rasqal_rowsource_reset(rasqal_rowsource* rowsource)
   if(rowsource->handler->reset)
     return rowsource->handler->reset(rowsource, rowsource->user_data);
 
-  return 0;
-}
-
-
-int
-rasqal_rowsource_set_preserve(rasqal_rowsource* rowsource, int preserve)
-{
-  if(rowsource->handler->set_preserve)
-    return rowsource->handler->set_preserve(rowsource,
-                                            rowsource->user_data, preserve);
-
+  if(rowsource->flags & RASQAL_ROWSOURCE_FLAGS_SAVED_ROWS) {
+    RASQAL_DEBUG3("%s rowsource %p resetting to use saved rows\n", 
+                  rowsource->handler->name, rowsource);
+    rowsource->offset = 0;
+  } else {
+    RASQAL_DEBUG3("WARNING: %s rowsource %p has no reset and there are no saved rows\n",
+                  rowsource->handler->name, rowsource);
+  }
   return 0;
 }
 
@@ -533,7 +636,7 @@ rasqal_rowsource_get_inner_rowsource(rasqal_rowsource* rowsource, int offset)
  *
  * If the user function @fn returns 0, the visit is truncated.
  *
- * Return value: 0 if the visit was truncated.
+ * Return value: non-0 if the visit was truncated or on failure.
  **/
 int
 rasqal_rowsource_visit(rasqal_rowsource* rowsource,
@@ -544,15 +647,24 @@ rasqal_rowsource_visit(rasqal_rowsource* rowsource,
   int offset;
   rasqal_rowsource* inner_rs;
   
+  if(!rowsource || !fn)
+    return 1;
+
   result = fn(rowsource, user_data);
-  if(result)
+  /* end if failed */
+  if(result < 0)
     return result;
 
+  /* Do not recurse if handled */
+  if(result > 0)
+    return 0;
+
   for(offset = 0;
       (inner_rs = rasqal_rowsource_get_inner_rowsource(rowsource, offset));
       offset++) {
     result = rasqal_rowsource_visit(inner_rs, fn, user_data);
-    if(result)
+    /* end if failed */
+    if(result < 0)
       return result;
   }
 
@@ -585,6 +697,49 @@ rasqal_rowsource_set_origin(rasqal_rowsource* rowsource,
 }
 
 
+static int
+rasqal_rowsource_visitor_set_requirements(rasqal_rowsource* rowsource,
+                                          void *user_data)
+{
+  unsigned int flags = *(unsigned int*)user_data;
+
+  if(rowsource->handler->set_requirements)
+    return rowsource->handler->set_requirements(rowsource, rowsource->user_data,
+                                                flags);
+
+  if(flags & RASQAL_ROWSOURCE_REQUIRE_RESET) {
+    if(!rowsource->handler->reset) {
+      /* If there is no reset handler, it is handled by this module and it is needed
+       * it is handled by this module
+       */
+      RASQAL_DEBUG3("setting %s rowsource %p to save rows\n",
+                    rowsource->handler->name, rowsource);
+      rowsource->flags |= RASQAL_ROWSOURCE_FLAGS_SAVE_ROWS;
+      return 1;
+    }
+  }
+
+  return 0;
+}
+
+
+int
+rasqal_rowsource_set_requirements(rasqal_rowsource* rowsource, unsigned int flags)
+{
+  return rasqal_rowsource_visit(rowsource, 
+                                rasqal_rowsource_visitor_set_requirements,
+                                &flags);
+}
+
+
+int
+rasqal_rowsource_request_grouping(rasqal_rowsource* rowsource)
+{
+  rowsource->generate_group = 1;
+
+  return 0;
+}
+
 
 #define SPACES_LENGTH 80
 static const char spaces[SPACES_LENGTH+1] = "                                                                                ";
@@ -606,11 +761,11 @@ rasqal_rowsource_write_internal(rasqal_rowsource *rowsource,
 {
   const char* rs_name = rowsource->handler->name;
   int arg_count = 0;
-  int indent_delta;
+  unsigned int indent_delta;
   int offset;
   rasqal_rowsource* inner_rowsource;
   
-  indent_delta = strlen(rs_name);
+  indent_delta = RASQAL_GOOD_CAST(unsigned int, strlen(rs_name));
 
   raptor_iostream_counted_string_write(rs_name, indent_delta, iostr);
   raptor_iostream_counted_string_write("(\n", 2, iostr);
@@ -663,6 +818,9 @@ rasqal_rowsource_print(rasqal_rowsource *rowsource, FILE* fh)
 {
   raptor_iostream *iostr;
 
+  if(!rowsource || !fh)
+    return;
+
   iostr = raptor_new_iostream_to_file_handle(rowsource->world->raptor_world_ptr, fh);
   rasqal_rowsource_write(rowsource, iostr);
   raptor_free_iostream(iostr);
@@ -692,7 +850,7 @@ main(int argc, char *argv[])
 
   for(i = 0; i < 4; i++) {
     rasqal_rowsource *rowsource;
-    size_t count;
+    int count;
 
     /* for _from_file */
     FILE *handle = NULL;
@@ -706,7 +864,7 @@ main(int argc, char *argv[])
         fprintf(stderr, "%s: Creating rowsource from a filename '%s'\n",
                 program, OUT_FILENAME);
 #endif
-        rowsource = rasqal_new_rowsource_from_filename((const char*)IN_FILENAME);
+        rowsource = rasqal_new_rowsource_from_filename(RASQAL_GOOD_CAST(const char*, IN_FILENAME));
         if(!rowsource) {
           fprintf(stderr, "%s: Failed to create rowsource to filename '%s'\n",
                   program, OUT_FILENAME);
@@ -718,7 +876,7 @@ main(int argc, char *argv[])
 #ifdef RASQAL_DEBUG
         fprintf(stderr, "%s: Creating rowsource from file handle\n", program);
 #endif
-        handle = fopen((const char*)OUT_FILENAME, "wb");
+        handle = fopen(RASQAL_GOOD_CAST(const char*, OUT_FILENAME), "wb");
         rowsource = rasqal_new_rowsource_from_file_handle(handle);
         if(!rowsource) {
           fprintf(stderr, "%s: Failed to create rowsource from a file handle\n", program);
@@ -757,7 +915,7 @@ main(int argc, char *argv[])
     count = rasqal_rowsource_get_rows_count(rowsource);
     if(count != OUT_BYTES_COUNT) {
       fprintf(stderr, "%s: I/O stream wrote %d bytes, expected %d\n", program,
-              (int)count, (int)OUT_BYTES_COUNT);
+              count, RASQAL_GOOD_CAST(int, OUT_BYTES_COUNT));
       return 1;
     }
     
@@ -782,7 +940,7 @@ main(int argc, char *argv[])
           return 1;
         }
         if(string_len != count) {
-          fprintf(stderr, "%s: I/O stream created a string length %d, expected %d\n", program, (int)string_len, (int)count);
+          fprintf(stderr, "%s: I/O stream created a string length %d, expected %d\n", program, RASQAL_BAD_CAST(int, string_len), count);
           return 1;
         }
         rasqal_free_memory(string);
