@@ -473,13 +473,13 @@ main(int argc, char *argv[])
   int end_column;
   int rc;
   raptor_sequence* triples;
-  rasqal_triples_source* triples_source;
+  rasqal_triples_source* triples_source = NULL;
   raptor_uri *base_uri = NULL;
   unsigned char *data_string = NULL;
   unsigned char *uri_string = NULL;
   /* <http://example.org#subject> <http://example.org#predicate> "object" . */
-#define SUBJECT_URI_STRING (const unsigned char*)"http://example.org#subject"
-#define PREDICATE_URI_STRING (const unsigned char*)"http://example.org#predicate"
+#define SUBJECT_URI_STRING RASQAL_GOOD_CAST(const unsigned char*, "http://example.org#subject")
+#define PREDICATE_URI_STRING RASQAL_GOOD_CAST(const unsigned char*, "http://example.org#predicate")
 #define OBJECT_STRING "object"
   raptor_uri* s_uri = NULL;
   raptor_uri* p_uri = NULL;
@@ -504,8 +504,8 @@ main(int argc, char *argv[])
   query = rasqal_new_query(world, "sparql", NULL);
   
   data_string = raptor_uri_filename_to_uri_string(data_file);
-  query_string = RASQAL_MALLOC(unsigned char*, strlen((const char*)data_string) + strlen(query_format) + 1);
-  sprintf((char*)query_string, query_format, data_string);
+  query_string = RASQAL_MALLOC(unsigned char*, strlen(RASQAL_GOOD_CAST(const char*, data_string)) + strlen(query_format) + 1);
+  sprintf(RASQAL_GOOD_CAST(char*, query_string), query_format, data_string);
   raptor_free_memory(data_string);
   
   uri_string = raptor_uri_filename_to_uri_string("");
@@ -586,7 +586,7 @@ main(int argc, char *argv[])
     o = row->values[2];
     if(!o ||
        (o && o->type != RASQAL_LITERAL_STRING) ||
-       strcmp((const char*)o->string, OBJECT_STRING)) {
+       strcmp(RASQAL_GOOD_CAST(const char*, o->string), OBJECT_STRING)) {
       fprintf(stderr, "%s: 'o' is bound to %s not string '%s'\n", program,
               rasqal_literal_as_string(o), OBJECT_STRING);
       failures++;
