@@ -1,10 +1,8 @@
+/* A Bison parser, made by GNU Bison 2.5.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
-
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -144,40 +142,43 @@
      SHA256 = 360,
      SHA384 = 361,
      SHA512 = 362,
-     EXPLAIN = 363,
-     LET = 364,
-     CURRENT_DATETIME = 365,
-     NOW = 366,
-     FROM_UNIXTIME = 367,
-     TO_UNIXTIME = 368,
-     HATHAT = 369,
-     SC_OR = 370,
-     SC_AND = 371,
-     EQ = 372,
-     NEQ = 373,
-     LT = 374,
-     GT = 375,
-     LE = 376,
-     GE = 377,
-     ASSIGN = 378,
-     STRING = 379,
-     LANG_TAG = 380,
-     DOUBLE_LITERAL = 381,
-     DOUBLE_POSITIVE_LITERAL = 382,
-     DOUBLE_NEGATIVE_LITERAL = 383,
-     INTEGER_LITERAL = 384,
-     INTEGER_POSITIVE_LITERAL = 385,
-     INTEGER_NEGATIVE_LITERAL = 386,
-     DECIMAL_LITERAL = 387,
-     DECIMAL_POSITIVE_LITERAL = 388,
-     DECIMAL_NEGATIVE_LITERAL = 389,
-     BOOLEAN_LITERAL = 390,
-     URI_LITERAL = 391,
-     URI_LITERAL_BRACE = 392,
-     QNAME_LITERAL = 393,
-     QNAME_LITERAL_BRACE = 394,
-     BLANK_LITERAL = 395,
-     IDENTIFIER = 396
+     UUID = 363,
+     STRUUID = 364,
+     VALUES = 365,
+     EXPLAIN = 366,
+     LET = 367,
+     CURRENT_DATETIME = 368,
+     NOW = 369,
+     FROM_UNIXTIME = 370,
+     TO_UNIXTIME = 371,
+     HATHAT = 372,
+     SC_OR = 373,
+     SC_AND = 374,
+     EQ = 375,
+     NEQ = 376,
+     LT = 377,
+     GT = 378,
+     LE = 379,
+     GE = 380,
+     ASSIGN = 381,
+     STRING = 382,
+     LANGTAG = 383,
+     DOUBLE_LITERAL = 384,
+     DOUBLE_POSITIVE_LITERAL = 385,
+     DOUBLE_NEGATIVE_LITERAL = 386,
+     INTEGER_LITERAL = 387,
+     INTEGER_POSITIVE_LITERAL = 388,
+     INTEGER_NEGATIVE_LITERAL = 389,
+     DECIMAL_LITERAL = 390,
+     DECIMAL_POSITIVE_LITERAL = 391,
+     DECIMAL_NEGATIVE_LITERAL = 392,
+     BOOLEAN_LITERAL = 393,
+     URI_LITERAL = 394,
+     URI_LITERAL_BRACE = 395,
+     QNAME_LITERAL = 396,
+     QNAME_LITERAL_BRACE = 397,
+     BLANK_LITERAL = 398,
+     IDENTIFIER = 399
    };
 #endif
 /* Tokens.  */
@@ -286,40 +287,43 @@
 #define SHA256 360
 #define SHA384 361
 #define SHA512 362
-#define EXPLAIN 363
-#define LET 364
-#define CURRENT_DATETIME 365
-#define NOW 366
-#define FROM_UNIXTIME 367
-#define TO_UNIXTIME 368
-#define HATHAT 369
-#define SC_OR 370
-#define SC_AND 371
-#define EQ 372
-#define NEQ 373
-#define LT 374
-#define GT 375
-#define LE 376
-#define GE 377
-#define ASSIGN 378
-#define STRING 379
-#define LANG_TAG 380
-#define DOUBLE_LITERAL 381
-#define DOUBLE_POSITIVE_LITERAL 382
-#define DOUBLE_NEGATIVE_LITERAL 383
-#define INTEGER_LITERAL 384
-#define INTEGER_POSITIVE_LITERAL 385
-#define INTEGER_NEGATIVE_LITERAL 386
-#define DECIMAL_LITERAL 387
-#define DECIMAL_POSITIVE_LITERAL 388
-#define DECIMAL_NEGATIVE_LITERAL 389
-#define BOOLEAN_LITERAL 390
-#define URI_LITERAL 391
-#define URI_LITERAL_BRACE 392
-#define QNAME_LITERAL 393
-#define QNAME_LITERAL_BRACE 394
-#define BLANK_LITERAL 395
-#define IDENTIFIER 396
+#define UUID 363
+#define STRUUID 364
+#define VALUES 365
+#define EXPLAIN 366
+#define LET 367
+#define CURRENT_DATETIME 368
+#define NOW 369
+#define FROM_UNIXTIME 370
+#define TO_UNIXTIME 371
+#define HATHAT 372
+#define SC_OR 373
+#define SC_AND 374
+#define EQ 375
+#define NEQ 376
+#define LT 377
+#define GT 378
+#define LE 379
+#define GE 380
+#define ASSIGN 381
+#define STRING 382
+#define LANGTAG 383
+#define DOUBLE_LITERAL 384
+#define DOUBLE_POSITIVE_LITERAL 385
+#define DOUBLE_NEGATIVE_LITERAL 386
+#define INTEGER_LITERAL 387
+#define INTEGER_POSITIVE_LITERAL 388
+#define INTEGER_NEGATIVE_LITERAL 389
+#define DECIMAL_LITERAL 390
+#define DECIMAL_POSITIVE_LITERAL 391
+#define DECIMAL_NEGATIVE_LITERAL 392
+#define BOOLEAN_LITERAL 393
+#define URI_LITERAL 394
+#define URI_LITERAL_BRACE 395
+#define QNAME_LITERAL 396
+#define QNAME_LITERAL_BRACE 397
+#define BLANK_LITERAL 398
+#define IDENTIFIER 399
 
 
 
@@ -328,8 +332,8 @@
 typedef union YYSTYPE
 {
 
-/* Line 1676 of yacc.c  */
-#line 140 "./sparql_parser.y"
+/* Line 2068 of yacc.c  */
+#line 143 "./sparql_parser.y"
 
   raptor_sequence *seq;
   rasqal_variable *variable;
@@ -354,8 +358,8 @@ typedef union YYSTYPE
 
 
 
-/* Line 1676 of yacc.c  */
-#line 359 "sparql_parser.tab.h"
+/* Line 2068 of yacc.c  */
+#line 363 "sparql_parser.tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
